myname = 'compare_ann_avg_v3.R'

mpostscript( myname, horiz=FALSE )

ppp_x = 3
ppp_y = 5

plots_per_page = ppp_x * ppp_y

source( '~/location.R' )
source('../mod_descrip_v10.R')
source('../make_all_filenames_v4.R')

source('~/src/mine/R/multi_page_png.R' )

seas = c( 'DJF', 'MAM', 'JJA', 'SON' )

#======================================================================================
do_mod_var = function( fn_all, modname, imod, varname_cmip5, varname_livneh, ivar, units_tag, seas ) {

	fname_mod_ds = paste0( '../', 
		fn_all$dstype[[IDSTYPE_16DEG]]$mod[[imod]]$var[[ivar]]$expt[[IEXPT_HISTORICAL]]$period[[IPERIOD_TRAINING_PERIOD]]$ds
		)

	nseas = length( seas )
	fname_mod = sub( 'srs.ds.nc', 'srs.ds.postdsbc.yearly.nc', fname_mod_ds )
	if( filesize( fname_mod ) < 1000 ) 
		stop(paste("File not found:", fname_mod ))
	print( fname_mod )

	if( varname_cmip5 == 'pr' ) {
		scalefact = 86400.	# convert kg/m**2/s to mm/day
		units = 'mm/day'
		offset = 0.0
		dv = make_discretevals( 'prism_precip', 
			c(0.2,0.4,0.6,0.8,1.0,2,3,4,5,6,8,10,14) )
		do_ratio = TRUE
		niters = 2
		}

	else if( varname_cmip5 == 'tasmin' ) {
		scalefact = 1.0
		offset = -273.15
		units = 'deg C'
		dv = make_discretevals( 'prism_temp', seq(-8,20,by=2))
		dv$values2label = seq(-8,20,by=4 )
		do_ratio = FALSE
		niters = 1
		}

	else if( varname_cmip5 == 'tasmax' ) {
		scalefact = 1.0
		offset = -273.15
		units = 'deg C'
		dv = make_discretevals( 'prism_temp', seq(4,34,by=2))
		dv$values2label = seq(4,34,by=4)
		do_ratio = FALSE
		niters = 1
		}

	else if( varname_cmip5 == 'DTR' ) {
		nztag = ''
		scalefact = 1.0
		offset = 0.0
		units = 'deg C'
		dv = make_discretevals( 'idl',
			seq(8,20,by=1))
		do_ratio = FALSE
		niters = 1
		}

	else
		stop(paste("uncaught case for var", varname_cmip5))

	fname_obs = paste0( '../../Livneh/livneh_NAmerExt_15Oct2014/',
			varname_livneh, '.1950-2005.', units_tag, '.crop.preproc_obs.', 
			varname_livneh, '_only.yearly.all.nc' )
	if( filesize( fname_obs ) < 1000 ) 
		stop(paste("File not found:", fname_obs ))
	print(fname_obs )

	ncid_mod = nc_open( fname_mod )
	ncid_obs = nc_open( fname_obs )

	mppng_source_file( fname_mod )

	dat_mod = (ncvar_get( ncid_mod ) + offset) * scalefact	# [nx,ny,nyears]
	dat_obs = (ncvar_get( ncid_obs ) + offset) * scalefact

	#--------------------
	# make yearly avg mod
	#--------------------
	dat_mod = mean2d( dat_mod )

	var_mod = ncid_mod$var[[1]]
	xvals_mod = var_mod$dim[[1]]$vals
	if( max( xvals_mod ) > 181 ) xvals_mod = xvals_mod - 360
	yvals_mod = var_mod$dim[[2]]$vals

	var_obs = ncid_obs$var[[1]]
	xvals_obs = var_obs$dim[[1]]$vals
	if( max( xvals_obs ) > 181 ) xvals_obs = xvals_obs - 360
	yvals_obs = var_obs$dim[[2]]$vals

	xlim = range( xvals_mod )
	ylim = range( yvals_mod )
	ylim[2] = 56

	cfh = 1.2
	cfy = 0.6

	ny_obs = length( yvals_obs )
	ny_mod = length( yvals_mod )

	col_map = grey(0.5)

	for( iter in 1:niters ) {

		if( iter == 2 ) {
			#------------------------------------------
			# On SECOND iteration, do precip in cm/year
			#------------------------------------------
			scalefact = 365.24 / 10	# convert from mm/day to cm/year
			units = 'cm/year'
			dat_obs = dat_obs * scalefact
			dat_mod = dat_mod * scalefact
			dv = make_discretevals( 'prism_precip', 
				c(10, 20, 30, 50, 80, 100, 120, 150, 180, 200, 300, 400) )
			do_ratio = FALSE
			dv_err_p_cm_yr = make_discretevals( 'pretty', seq(-12, 12, by=2))
			}

		main = paste0(varname_cmip5, ' [', units, ']' )
		print(main)

		ccontour( xvals_obs, yvals_obs, dat_obs,
			raster=TRUE,
			xlim=xlim, ylim=ylim, 
			discretevals=dv,
			contour_show=FALSE,
			cbar_fudge_height=cfh,
			cbar_fudge_yoffset=cfy,
			main=paste('Obs mean',main) )
		map('state', add=TRUE, col=col_map )
		mppng_next_frame()

		ccontour( xvals_mod, yvals_mod, dat_mod,
			raster=TRUE,
			xlim=xlim, ylim=ylim, 
			discretevals=dv,
			contour_show=FALSE,
			cbar_fudge_yoffset=cfy,
			cbar_fudge_height=cfh,
			main=paste(modname,'mean',main) )
		map('state', add=TRUE, col=col_map )
		mppng_next_frame()

		if( do_ratio ) {
			rv = calc_2d_ratio( xvals_mod, yvals_mod, dat_mod,
				xvals_obs, yvals_obs, dat_obs,
				diffratio_pct=TRUE )
			main = 'Model error w.r.t. obs [%]'
			dv_err = make_discretevals( 'pretty', seq(-25,25,by=5))
			dv_err$values2label = seq(-25,25,by=10 )
			}
		else
			{
			rv = calc_2d_ratio( xvals_mod, yvals_mod, dat_mod,
				xvals_obs, yvals_obs, dat_obs,
				just_difference=TRUE )
			if( iter == 1 ) {
				dv_err = make_discretevals( 'pretty', seq(-5.5,5.5,by=1))
				main = 'Model error w.r.t. obs [degC]'
				}
			else
				{
				dv_err = dv_err_p_cm_yr
				main = 'Model error w.r.t. obs [cm/yr]'
				}
			}

		ccontour( rv$xvals, rv$yvals, rv$ratio,
			raster=TRUE,
			xlim=xlim, ylim=ylim, 
			discretevals=dv_err,
			contour_show=FALSE,
			cbar_fudge_height=cfh,
			cbar_fudge_yoffset=cfy,
			main=main )
		map('state', add=TRUE, col=col_map )

		terr = rv$ratio
		terr[ is.nan     ( terr ) ] = NA
		terr[ is.infinite( terr ) ] = NA
		merr = mean( terr, na.rm=TRUE )
		rmse = sqrt( mean( terr*terr, na.rm=TRUE ))

		str = paste0( 'RMSE: ', mformatC('%.1f',rmse), '  Mean: ', mformatC('%.1f',merr))
		print(str)
		text( min(xlim), max(ylim), str, adj=c(0,1.3), cex=1.2 )

		mppng_next_frame()
		}

	nc_close( ncid_mod )
	nc_close( ncid_obs )
}

#======================================================================================
doitfor = function( fn_all, modd, modname, imod, seas ) {

	nvar = length( modd$var )

	for( ivar in 1:nvar ) {
		varname_cmip5  = modd$var[[ivar]]$varname_cmip5  
		varname_livneh = modd$var[[ivar]]$varname_livneh 
		units_tag      = modd$var[[ivar]]$units_tag
		do_mod_var( fn_all, modname, imod, varname_cmip5, varname_livneh, ivar, units_tag, seas )
		}
}

#======================================================================================
calc_dat = function( opts ) {

	dat = list()

        modd = make_mod_descrip( location, LOC_NASA, LOC_CIRRUS )
	nmod = length( modd$model )

	#-----------------------------
	# Get flat list of model names
	#-----------------------------
	for( imod in 1:nmod ) {
		if( imod == 1 )
			modnames = modd$model[[imod]]$modname_cmip5
		else
			modnames = c( modnames, modd$model[[imod]]$modname_cmip5 )
		}

	#---------------------------
	# Get flat list of var names
	#---------------------------
	nvar = length( modd$var )
	for( ivar in 1:nvar ) {
		if( ivar == 1 ) 
			varnames = modd$var[[ivar]]$varname_cmip5
		else
			varnames = c( varnames, modd$var[[ivar]]$varname_cmip5 )
		}

	hierarchy = list( modnames )

	npages_tot = nmod*nvar

	#---------------------------------------------------
	# Set up multiple png plots per page in a html bunch
	#---------------------------------------------------
	multi_page_png( myname, plots_per_page, npages_tot,
		width=700, height=1000,
		header_html='Annual average of Livneh Obs vs. LOCA downscaled (1/16 degree)',
		hierarchy=hierarchy )

	#-----------------------------------------------
	# Must do 'par' AFTER the call to multi_page_png
	#-----------------------------------------------
	par(mfrow=c(ppp_y,ppp_x), mar=c(2.7,1,2.0,0.5), mgp=c(3,0,0))

	fn_all = make_all_filenames( modd )

print(paste('nmod=', nmod))
	for( imod in 1:nmod ) {
		doitfor( fn_all, modd, modd$model[[imod]]$modname_cmip5, imod, seas )
		}

	return(dat)
}

#======================================================================================
plotup = function( dat, opts, po ) {

}

#======================================================================================
doit = function() {

	use_cache = FALSE
	cache_fname = paste(myname,'_Rcache', sep='' )
	cache_file_exists = (filesize(cache_fname) > 10)

	#-----------------
	# Set data options
	#-----------------
	opts = list()

	#--------------------
	# Use cache, if asked
	#--------------------
	if( use_cache ) {
		if( cache_file_exists ) {
			print(paste("loading data from cache file", cache_fname))
			load( file=cache_fname )
			print('...done')
			do_calc_dat = FALSE
			}
		else
			do_calc_dat = TRUE
		}
	else
		do_calc_dat = TRUE

	#-------------------
	# Calculate the data
	#-------------------
	if( do_calc_dat )
		dat = calc_dat( opts )
		
	#-----------------------------------------------------
	# Save to cache if requested and doesn't exist already
	#-----------------------------------------------------
	if( use_cache && (! cache_file_exists))
		save( file=cache_fname, dat, opts )
			
	#---------------------------------
	# Set plot options and plot result
	#---------------------------------
	po = list()
	plotup( dat, opts, po )
}

#======================================================================================

doit()
mdev.off( myname )
